(*************************************************************************
*                                                                        *
*    Package  MATHEMATIK - PACK  from unit  NUMERIK                      *
*                                                                        *
**************************************************************************
    *                                                                    *
    *  J. Desteffani IA85                     01.01.1987 - 26.01.1987    *
    *                                                                    *
    *  p u r p o s e :                                                   *
    *    Enthaelt die Algorithmen fuer die verschiedensten Matrizen -    *
    *    berechnungen.                                                   *
    *                                                                    *
    *  e n t r i e s :                                                   *
    *    PROCEDURE Matrix_Mult       (VAR a, b, res      :T_Matrix;      *
    *                                     n1, m1, n2, m2 :integer;);     *
    *                                                                    *
    *    PROCEDURE Matrix_Eye        (VAR a              :T_Matrix;      *
    *                                     n              :integer);      *
    *                                                                    *
    *    PROCEDURE LU_Decomp         (VAR a, pvt         :T_Matrix;      *
    *                                     n              :integer;       *
    *                                 VAR f              :integer;       *
    *                                 VAR singulaer      :boolean);      *
    *                                                                    *
    *    PROCEDURE Solve             (VAR a, b           :T_Matrix;      *
    *                                     n              :integer);      *
    *                                                                    *
    *    PROCEDURE QR_Decomp         (VAR a, b           :T_Matrix);     *
    *                                     n, m           :integer;       *
    *                                 VAR singulaer      :boolean);      *
    *                                                                    *
    *    PROCEDURE Matrix_Inverse    (VAR a, pvt         :T_Matrix;      *
    *                                     n              :integer;       *
    *                                 VAR singulaer      :boolean);      *
    *                                                                    *
    **********************************************************************)

PROCEDURE Matrix_Mult (VAR a, b, res :T_Matrix; n1, m1, n2, m2 :integer);

   VAR
      i, j, k     :integer;
      Temp        :real;
      M_Temp      :T_Matrix;

BEGIN
   N_Message (20);
   FOR i:= 1 TO n1 DO BEGIN
      Message_integer_1:= i;
      N_Message (21);
      FOR j:= 1 TO m2 DO BEGIN
         Temp:= 0;
         FOR k:= 1 TO n2 DO BEGIN
            Temp:= Temp + a[i,k] * b[k,j];
         END;
         M_Temp[i,j]:= Temp;
      END;
   END;
   res:= M_Temp;
END;


(*------------------------------------------------------------------------*)

PROCEDURE Matrix_Eye (VAR a :T_Matrix; n: integer);

   VAR
      i, j     :integer;

BEGIN
   FOR i:= 1 TO n DO BEGIN
      FOR j:= 1 TO n DO BEGIN
         IF i = j
         THEN a[i,j]:= 1
         ELSE a[i,j]:= 0;
      END;
   END;
END;

(*------------------------------------------------------------------------*)

PROCEDURE LU_Decomp (VAR a, pvt    :T_Matrix; n :integer; VAR f :integer;
                     VAR singulaer :boolean);

   VAR
      i, j, k, jmax    :integer;
      Pivot_Element    :real;
      h                :T_Vektor;

BEGIN
   f:= 1;
   Matrix_Eye (pvt, n);
   singulaer:= false;
   N_Message (20);
   Message_integer_1:= 1;
   N_Message (21);
   IF n > 1 THEN BEGIN
      FOR i:= 1 TO n DO BEGIN
         Message_integer_1:= i;
         N_Message (21);
         jmax:= i;
         IF Pivot_Enable THEN BEGIN
            FOR j:= i+1 TO n DO BEGIN
               IF abs (a[j,i]) > abs (a[jmax,i]) THEN jmax:= j;
            END;
         END;
         Pivot_Element:= a[jmax,i];
         IF abs(Pivot_Element) > Epsilon THEN BEGIN
            IF jmax <> i THEN BEGIN
               h:= a[i];
               a[i]:= a[jmax];
               a[jmax]:= h;
               h:= pvt[i];
               pvt[i]:= pvt[jmax];
               pvt[jmax]:= h;
               f:= -f;
            END;
            FOR k:= i+1 TO n DO BEGIN
               IF a[k,i] <> 0 THEN BEGIN
                  a[k,i]:= a[k,i]/Pivot_Element;
                  FOR j:= i+1 TO n DO a[k,j]:= a[k,j]-a[k,i]*a[i,j];
               END;
            END;
         END
         ELSE BEGIN
            singulaer:= true;
         END;
      END;
   END;
   singulaer:= (singulaer) or (a[n,n] = 0);
 END;

(*------------------------------------------------------------------------*)

PROCEDURE Solve (VAR a, b :T_Matrix; n :integer);

   VAR
      i, k    :integer;
      s       :real;

BEGIN
   IF n > 1 THEN BEGIN
      FOR k:= 1 TO n-1 DO BEGIN
         s:= b[k,1];
         FOR i:= k+1 TO n DO b[i,1]:= b[i,1]-a[i,k]*s;
      END;
      b[n,1]:= b[n,1]/a[n,n];
      FOR k:= n-1 DOWNTO 1 DO BEGIN
         FOR i:= k+1 TO n DO b[k,1]:= b[k,1]-a[k,i]*b[i,1];
         b[k,1]:= b[k,1]/a[k,k];
      END;
   END
   ELSE BEGIN
      b[1,1]:= b[1,1]/a[1,1];
   END;
END;

(*------------------------------------------------------------------------*)

PROCEDURE QR_Decomp (VAR a, b :T_Matrix;  n, m  :integer;
                     VAR singulaer :boolean);

   VAR
      i, j, k        :integer;
      h, cot, si, co :real;
      save1, save2   :real;

BEGIN
   Matrix_Eye (b, n);
   singulaer:= false;;
   i:= 1;
   N_Message (20);
   Message_integer_1:= i;
   N_Message (21);
   REPEAT
      FOR k:= i+1 TO n DO BEGIN
         IF a[k,i] <> 0 THEN BEGIN
            cot:= a[i,i]/a[k,i];
            si:= 1/sqrt(1 + cot*cot);
            co:= si*cot;
            a[i,i]:= a[i,i]*co + a[k,i]*si;
            FOR j:= i+1 TO n DO BEGIN
               h:= a[i,j]*co + a[k,j]*si;
               a[k,j]:= -a[i,j]*si + a[k,j]*co;
               a[i,j]:= h;
            END;
            FOR j:= 1 TO n DO BEGIN
               save1:= co*b[j,i] + si*b[j,k];
               save2:= -si*b[j,i] + co*b[j,k];
               b[j,i]:= save1;
               b[j,k]:= save2;
            END;
         END;
         a[k,i]:= 0;
      END;
      IF a[i,i] = 0 THEN BEGIN
         singulaer:= true;
      END
      ELSE BEGIN
         i:= i+1;
      END;
      Message_integer_1:= i;
      N_Message (21);
   UNTIL (i=n) or singulaer;
   singulaer:= singulaer or (a[n,n] = 0);
END;

(*------------------------------------------------------------------------*)

PROCEDURE Matrix_Inverse (VAR a, pvt :T_Matrix; n :integer;
                          VAR singulaer :boolean);

   VAR
      i, j, k, jmax   :integer;
      Pivot_Element   :real;
      h               :T_Vektor;

BEGIN
   singulaer:= false;
   Matrix_Eye (pvt, n);
   N_Message(20);
   FOR i:= 1 TO n DO BEGIN
      Message_integer_1:= i;
      N_Message (21);
      jmax:= i;
      IF Pivot_Enable THEN BEGIN
         FOR j:= i+1 TO n DO BEGIN
            IF abs (a[j,i]) > abs (a[jmax,i]) THEN jmax:= j;
         END;
      END;
      Pivot_Element:= a[jmax,i];
      IF abs(Pivot_Element) > Epsilon THEN BEGIN
         IF jmax <> i THEN BEGIN
            h:= a[i];
            a[i]:= a[jmax];
            a[jmax]:= h;
            h:= pvt[i];
            pvt[i]:= pvt[jmax];
            pvt[jmax]:= h;
         END;
         FOR j:= 1 TO n DO BEGIN
            FOR k:= 1 TO n DO BEGIN
                IF (j <> i) and (k <> i) THEN BEGIN
                   a[j,k]:= a[j,k]*a[i,i] - a[j,i]*a[i,k];
               END;
            END;
         END;
         a[i,i]:= 1;
         FOR j:= 1 TO n DO BEGIN
            IF (j <> i) THEN a[i,j]:= -a[i,j];
         END;
         FOR j:= 1 TO n DO BEGIN
            FOR k:= 1 TO n DO a[j,k]:= a[j,k]/Pivot_Element;
         END;
      END
      ELSE BEGIN
         N_Error (60);
         exit;
      END;
   END;
   Message_Enable:= false;
   Matrix_Mult (a, pvt, a, n, n, n, n);
   Message_Enable:= true;
END;