(*************************************************************************
*                                                                        *
*                                                                        *
*    p r o g r a m      NUMERIK               Version  2.01              *
*                                                                        *
*                                                                        *
**************************************************************************
    *                                                                    *
    *                                                                    *
    *  J. Desteffani IA85                     27.12.1986 - 09.02.1987    *
    *                                                                    *
    *                                                                    *
    *  p u r p o s e :                                                   *
    *                                                                    *
    *  In diesem Programm sollen einmal die wichtigsten numerischen      *
    *  Algorithmen verwirklicht sein.                                    *
    *  Gesteuert wird dieses Programm mittels Befehlsaufrufen und nicht  *
    *  durch langweilige Menue's. (Ausnahme: plot)                       *
    *  Eine HELP Anweisung steht zur Verfuegung.                         *
    *  Dieses Programm soll auf der PR1ME and der HTL nicht unbedingt    *
    *  lauffaehig sein.                                                  *
    *                                                                    *
    *                   ---  ZIELE  ---                                  *
    *                                                                    *
    *  VERSION 1.00  - die Berechnung der Determinante                   *
    *                - sonstige bereits bestehende Programme             *
    *                - Filemanagement                                    *
    *                - Eingabeauswertungsproceduren bewerkstelligen      *
    *                                                                    *
    *  VERSION 2.00: - weitere Numerikproceduren implementieren          *
    *                - plot-Procedure                                    *
    *                - Ordnung im Programm schaffen                      *
    *                - Fehlermeldungen, Meldungen zentralisieren         *
    *                                                                    *
    *  VERSION 2.10: - plot-Procedure verbessern (mehrere Kurven auf     *
    *                  derselben Graphik.)                               *
    *                - Wunsch Mister Benz: [return] = no.                *
    *                                                                    *
    **********************************************************************)


PROGRAM Numerik;

   (**********************************************************************
   *                                                                     *
   *        ENTER & SCREEN  global agreements   (Interface)              *
   *                                                                     *
   ***********************************************************************)

   CONST
      Enter_Line  = 25;
      max_Enter   = 80;
      max_Message = 80;

   TYPE
      T_Message_Str    = String [max_Enter];
      T_Enter_String   = String [max_Message];
      T_char_Menge     = SET OF char;
      T_Typenmenge     = SET OF (charT, integerT, realT, StringT,
                                booleanT, NullT);
      T_Typenrecord    = RECORD
                            charX          :char;
                            integerX       :integer;
                            realX          :real;
                            StringX        :T_Enter_String;
                            booleanX       :boolean;
                         END;

   (**********************************************************************
   *                                                                     *
   *         globale Variabeln   N U M E R I K                           *
   *                                                                     *
   ***********************************************************************)

   CONST
       minBefLaenge      =   1;
       maxBefLaenge      =   8;
       minArgLaenge      =   1;
       maxArgLaenge      =   8;
       AnzArg            =   3;
       max_Ind           =  50;
       max_Ind_Tabelle   = 200;
       Matrix_Call       = '.MTX';
       Result_Name       = 'res';
       Table_Name        = 'table';
       Name_L_Matrix     = 'l';
       Name_U_Matrix     = 'u';
       Name_Q_Matrix     = 'q';
       Name_R_Matrix     = 'r';
       Name_Pivot_Matrix = 'pivot';

   TYPE
       T_Vektor         = ARRAY [1..max_Ind] OF real;
       T_Matrix         = ARRAY [1..max_Ind] OF T_Vektor;
       T_integer_Vektor = ARRAY [1..max_Ind] OF integer;
       T_Matrix_Zeiger  = ^T_Matrix_Wurzel;
       T_Daten_Zeiger   = ^T_Daten;
       T_Zahl_Typen     = (Zreal, Zinteger, word);
       T_Format         = (kurz, lang, lang_exp, kurz_exp);

       T_Daten          = RECORD
                             next   :T_Daten_Zeiger;
                             Zahl   :real;
                          END;

       T_Matrix_Wurzel  = RECORD
                             vor,next :T_Matrix_Zeiger;
                             Name     :T_Enter_String;
                             n, m     :integer;
                             Wurzel   :T_Daten_Zeiger;
                          END;

       T_Befehl         = (add, aitken,
                          cholesky, clear,
                          del, det, display,
                          edit, eps, exit, eye, euler,
                          help,
                          input, inv,
                          lagrange, lin, list, load, long, lr, lu,
                          mult,
                          no,
                          pivot, plot, polytab, prim,
                          qr,
                          rand, rank, report, result,
                          save, short, spline, sr, status, store, sub,
                          transp);

       T_Eingabe        = RECORD
                             Satz  :T_Enter_String;
                             Art   :integer;
                             Bef   :ARRAY [1..AnzArg] OF T_Befehl;
                             Nr    :ARRAY [1..AnzArg] OF T_Matrix_Zeiger;
                             Arg   :ARRAY [1..AnzArg] OF T_Enter_String;
                             Zahl  :ARRAY [1..AnzArg] OF T_Zahl_Typen;
                             r     :ARRAY [1..AnzArg] OF real;
                             i     :ARRAY [1..AnzArg] OF integer;
                          END;

      T_Matrix_Intern   = RECORD
                             Name  :T_Enter_String;
                             n, m  :integer;
                             x     :T_Matrix;
                          END;

       T_Tabelle        = RECORD
                             Name  :T_Enter_String;
                             n, m  :integer;
                             x     :ARRAY [1..max_Ind_Tabelle, 1..2] of real;
                          END;


   VAR
       Report_Enable     :boolean;
       Pivot_Enable      :boolean;
       Result_Enable     :boolean;
       Message_Enable    :boolean;
       leave             :boolean;
       new_run           :boolean;
       Message_integer_1 :integer;
       Message_integer_2 :integer;
       Message_real_1    :real;
       input_boolean     :boolean;
       input_real        :real;
       Message_String_1  :T_Message_Str;
       Epsilon           :real;
       Error_Arg         :T_Enter_String;
       Format            :T_Format;
       Matrizen          :T_Matrix_Zeiger;
       Eingabe           :T_Eingabe;
       T1                :T_Tabelle;
       M1, M2, M3        :T_Matrix_Intern;

   (***********************************************************************
   *                                                                      *
   *         Proceduren   E N T E R   &   S C R E E N                     *
   *                                                                      *
   ************************************************************************)

   {$I ENTER.PAS}
(*
   {$I ENTERH.PAS}
*)
   (***********************************************************************
   *                                                                      *
   *         Proceduren   N U M E R I K                                   *
   *                                                                      *
   ************************************************************************)

   PROCEDURE title;                                     FORWARD;
   PROCEDURE list_matrix (x_matrix :T_Matrix_Zeiger);   FORWARD;
   PROCEDURE N_Error (numbre: integer);                 FORWARD;

(*------------------------------------------------------------------------*)

   PROCEDURE Window_on;
   BEGIN
      Window (1,3,80,23);
   END;

   PROCEDURE Window_off;
   BEGIN
      Window (1,1,80,25);
   END;

(*------------------------------------------------------------------------*)

   PROCEDURE clrMatrix (Matrix: T_Matrix_Wurzel);
   BEGIN
      WITH Matrix DO BEGIN
         vor:= nil;
         next:= nil;
         Name:= '';
         n:= 0;
         m:= 0;
         Wurzel:= nil;
      END;
   END;

   PROCEDURE clrEingabe;
      VAR
         j  :integer;
   BEGIN
      WITH Eingabe DO BEGIN
         FOR j:= 1 TO AnzArg DO BEGIN
            Bef[j]:= no;
            Arg[j]:= '';
            Nr[j] := nil;
            Zahl[j]:= word;
            r[j]:= 0.0;
            i[j]:= 0;
         END;
         Art:= 0;
      END;
   END;

(*------------------------------------------------------------------------*)

   PROCEDURE write_formated (Zahl :real);
   BEGIN
      CASE Format OF
      kurz     : BEGIN
                    IF (abs (Zahl) < 1e2)
                    THEN write (Zahl:8:4)
                    ELSE write (Zahl:8);
                 END;
      lang     : BEGIN
                    IF (abs (Zahl) < 1e4)
                    THEN write (Zahl:16:10)
                    ELSE write (Zahl:16);
                 END;
      kurz_exp : write (Zahl:8);
      lang_exp : write (Zahl:16);
      END;
   END;

(*------------------------------------------------------------------------*)
(*
   FUNCTION Befehl_Test (Word: T_Enter_String): T_Befehl; BEGIN END;
   FUNCTION Art_Test (Befehl: T_Befehl): integer; BEGIN END;
   FUNCTION Argument_Test (Argument: T_Enter_String): T_Matrix_Zeiger; BEGIN END;
   FUNCTION Zahl_Test (Argument: T_Enter_String; VAR r: real; VAR i: integer): T_Zahl_Typen; BEGIN END;
   PROCEDURE get_Befehl (VAR x: T_Eingabe); BEGIN END;
*)

   {$I INPTEST.PAS}

(*------------------------------------------------------------------------*)


   PROCEDURE N_Error; (* number: integer *)
   BEGIN
      writeln;
      CASE numbre OF
       10: write ('syntax error');
       20: write ('underflow error');
       21: write ('overflow error');
       25: write ('argument too low');
       26: write ('argument too high');
       28: write ('impossible application');
       40: write ('argument error');
       42: write ('argument must be integer');
       44: write ('argument must be naturally');
       45: write ('undefined argument (', Error_Arg, ')');
       47: write ('impossible argument');
       49: write ('argument (', Error_Arg, ') too long error');
       60: write ('matrix must be square');
       62: write ('matrix dimension error');
       63: write ('matrix is singulare');
       65: write ('matrix not positiv oriented');
       66: write ('matrix is not symmetrical, take upper right triangle-matrix');
       67: write ('matrix must have 2 columns');
       68: write ('too less points for interpolation');
       69: write ('matrix must be in rising order');
       70: write ('two equal points');
       71: write ('points must be entered in rising order');
       80: write ('matrix (', Error_Arg, ') is not a table, taken the first 2 columns');
       81: write ('matrix (', Error_Arg, ') is a table, taken the first 50 lines');
      110: write ('can`t find file on disk');
      111: write ('can`t find help.txt file on disk');
      112: write ('impossible to find help - information');
      210: write ('storage is empty');
      211: write ('storage is already empty');
      220: write ('no result found in storage');
      510: write ('file read error [', Error_Arg, ']');
      520: write ('intern pointer error');
      530: write ('program error, call system operator');
      END;
      Error_Arg:='';
      write ('  ');
   END;

(*------------------------------------------------------------------------*)

   PROCEDURE N_Message (numbre :integer);

      VAR
         wait           :boolean;
         Index, Cx, Cy  :integer;

   BEGIN
      IF (not Message_Enable) THEN exit;
      IF (not Report_Enable) and ((Numbre < 30) and (Numbre >= 20)) THEN exit;
      IF (Numbre = 1) and
         (not (Eingabe.Bef[1] in [add, aitken, lr, mult, prim, save, store, sub])) THEN BEGIN
         WITH Eingabe DO BEGIN
            write (Arg[1]);
            FOR Index:= 2 TO AnzArg DO BEGIN
               IF Arg[Index] <> '' THEN write (' (', Arg[Index], ')');
            END;
         END;
         exit;
      END;
      IF (Numbre = 1) THEN BEGIN
         CASE Eingabe.Bef[1] OF
            add    :write ('(', Eingabe.Arg[2], ') + (', Eingabe.Arg[3], ')');
            aitken :write ('interpolation  Aitken - Neville');
            save   :write ('save');
            store  :write ('(', Eingabe.Arg[3], ') = (', Eingabe.Arg[2], ')');
            lr     :write ('list (', Result_Name, ')');
            mult   :write ('(', Eingabe.Arg[2], ') * (', Eingabe.Arg[3], ')');
            prim   :write ('prim (', Eingabe.Arg[2], ') ?  ');
            sub    :write ('(', Eingabe.Arg[2], ') - (', Eingabe.Arg[3], ')');
         END;
         exit;
      END;
      IF (Numbre = 9) THEN BEGIN
         IF not Result_Enable THEN exit;
         CASE Eingabe.Bef[1] OF
         add, det, cholesky, inv, lin, mult, sub, transp
            :List_Matrix (Argument_Test (Result_Name));
         edit, input, rand
            :List_Matrix (Argument_Test (Eingabe.Arg[2]));
         lu :BEGIN writeln; write ('result is stored in (l), (u) and (pivot)'); END;
         qr :BEGIN writeln; write ('result is stored in (q) and (r)'); END;
         lagrange, spline, polytab
            :BEGIN writeln; write ('table of points is saved in (', Result_Name, ')'); END;
         END;
         exit;
      END;
      CASE Numbre OF
          0 :writeln;
         10 :write (' YES');
         11 :write (' NO');
         12 :write (' OK');
         13 :write ('                                     ');
         16 :write (' (', Message_String_1, ')');
         19 :BEGIN Cx:= whereX; Cy:= whereY;
             write (' - index [', Message_integer_1, ',', Message_integer_2, ']');
             gotoXY (Cx,Cy); END;
         20 :write (' :');
         21 :BEGIN Cx:= whereX; Cy:= whereY; write (Message_integer_1);
             gotoXY (Cx,Cy); END;
         40 :BEGIN writeln; write ('storage is saved'); END;
         41 :BEGIN writeln; write ('storage is deleted'); END;
         50 :write_Formated (Message_real_1);
         51 :BEGIN writeln; write ('kleinster Teiler:', Message_real_1:6:0); END;
         52 :BEGIN writeln; write ('Neustelle: ');
             write_Formated (Message_real_1); END;
         90 :BEGIN writeln; write ('sorry, this procedure is not ready !'); END;
         91 :BEGIN writeln; write ('this procedure not available (test - version'); END;
         98 :BEGIN Window_off; get_Keypressed ('press key'); Window_on; END;
         99 :BEGIN gotoXY (25,10); write ('THANK`S  A  LOT,  GOODBYE !'); END;
      END;
   END;

(*------------------------------------------------------------------------*)

   FUNCTION N_Ask (Numbre :integer): boolean;

      VAR
         x   :boolean;

   BEGIN
      x:= false;
      Window_off;
      CASE Numbre OF
        1: BEGIN get_boolean_inkl ('more ?  ', input_boolean, x);
           x:= (input_boolean) and (x); END;
        2: x:= get_boolean ('exit, are you sure ?  ');
        3: x:= get_boolean ('file (' + Message_String_1 + ') already exists, overwrite ?  ');
        6: x:= get_boolean ('ready to delete storage ?  ');
        7: x:= get_boolean ('ready to save storage on disk ?  ');
        8: x:= get_boolean ('break input ?  ');
       30: get_real_inkl ('>> ', input_real, x);
       35: get_real_inkl ('Stuetzpunkt (x): ', input_real, x);
       50: get_real_inkl ('min - (x): ', input_real, x);
       51: get_real_inkl ('max - (x): ', input_real, x);
       52: get_real_inkl ('min - (y): ', input_real, x);
       53: get_real_inkl ('max - (y): ', input_real, x);
      END;
      N_Ask:= x;
      Window_on;
   END;

(*------------------------------------------------------------------------*)

   FUNCTION N_integer_Ask (numbre  :integer): integer;

      VAR
         x  :integer;

   BEGIN
      Window_off;
      CASE numbre OF
       10: x:= get_m_integer ('number of points: ', 2, Max_Ind_Tabelle);
       11: x:= get_m_integer ('lines: ', 1, max_Ind);
       12: x:= get_m_integer ('columns: ', 1, max_Ind);
       13: x:= get_m_integer ('line: ', 1, max_Ind);
       14: x:= get_m_integer ('column: ', 1, max_Ind);
       15: x:= get_m_integer ('lines/columns: ', 1, max_Ind);
      END;
      Window_on;
      N_integer_Ask:= x;
   END;

(*------------------------------------------------------------------------*)

   FUNCTION N_real_Ask (numbre  :integer): real;

      VAR
         x   :real;

   BEGIN
      Window_off;
      CASE numbre OF
       10: x:= get_real ('lowest point (x): ');
       11: x:= get_real ('highest point (x): ');
       20: x:= get_real ('Neustelle: ');
       21: x:= get_real ('Stuetzwert (y): ');
      END;
      Window_on;
      N_real_Ask:= x;
   END;

(*------------------------------------------------------------------------*)

   FUNCTION N_char_Ask (numbre  :integer): char;

      VAR
         x   :char;

   BEGIN
      Window_off;
      CASE numbre OF
       10: x:= get_char ('enter plot-character: ');
       20: x:= get_m_char ('your choose ?  ', ['a', 'o', 't', 'p', 'e']);
      END;
      Window_on;
      N_char_Ask:= x;
   END;

(*------------------------------------------------------------------------*)

   FUNCTION N_String_Ask (numbre  :integer): T_Enter_String;

      VAR
         x:   T_Enter_String;

   BEGIN
      Window_off;
      CASE numbre OF
       10: x:= get_m_String ('new name:  ', maxArgLaenge);
       15: x:= get_m_String ('enter table name: ', maxArgLaenge);
      END;
      Window_on;
      N_String_Ask:= x;
   END;

(*------------------------------------------------------------------------*)
(*
   PROCEDURE del_Intern (Zeiger: T_Matrix_Zeiger); BEGIN END;
   PROCEDURE load_Intern (Zeiger: T_Matrix_Zeiger; VAR Feld: T_Matrix_Intern); BEGIN END;
   PROCEDURE save_Intern (VAR Feld: T_Matrix_Intern); BEGIN END;
   FUNCTION get_Matrix_Zeiger (Name: T_Enter_String): T_Matrix_Zeiger; BEGIN END;
*)

   {$I INTERN.PAS}

(*------------------------------------------------------------------------*)

   {$I MATPACK.PAS}

(*------------------------------------------------------------------------*)

(*
   PROCEDURE do_add; BEGIN N_Message (91) END;
   PROCEDURE do_aitken; BEGIN N_Message (91) END;
   PROCEDURE do_cholesky; BEGIN N_Message (91) END;
   PROCEDURE do_clear; BEGIN N_Message (91) END;
   PROCEDURE do_del; BEGIN N_Message (91) END;
   PROCEDURE do_det; BEGIN N_Message (91) END;
   PROCEDURE do_display; BEGIN N_Message (91) END;
   PROCEDURE do_edit; BEGIN N_Message (91) END;
   PROCEDURE do_eps; BEGIN N_Message (91) END;
   PROCEDURE do_euler; BEGIN N_Message (91) END;
   PROCEDURE do_exit; BEGIN N_Message (91) END;
   PROCEDURE do_eye; BEGIN N_Message (91) END;
   PROCEDURE do_help; BEGIN N_Message (91) END;
   PROCEDURE do_input; BEGIN N_Message (91) END;
   PROCEDURE do_inv; BEGIN N_Message (91) END;
   PROCEDURE do_lagrange; BEGIN N_Message (91) END;
   PROCEDURE do_lin; BEGIN N_Message (91) END;
   PROCEDURE do_list; BEGIN N_Message (91) END;
   PROCEDURE do_load; BEGIN N_Message (91) END;
   PROCEDURE do_long; BEGIN N_Message (91) END;
   PROCEDURE do_lr; BEGIN N_Message (91) END;
   PROCEDURE do_lu; BEGIN N_Message (91) END;
   PROCEDURE do_mult; BEGIN N_Message (91) END;
   PROCEDURE do_pivot; BEGIN N_Message (91) END;
   PROCEDURE do_plot; BEGIN N_Message (91) END;
   PROCEDURE do_polytab; BEGIN N_Message (91) END;
   PROCEDURE do_prim; BEGIN N_Message (91) END;
   PROCEDURE do_qr; BEGIN N_Message (91) END;
   PROCEDURE do_rand; BEGIN N_Message (91) END;
   PROCEDURE do_rank; BEGIN N_Message (91) END;
   PROCEDURE do_report; BEGIN N_Message (91) END;
   PROCEDURE do_result; BEGIN N_Message (91) END;
   PROCEDURE do_save; BEGIN N_Message (91) END;
   PROCEDURE do_short; BEGIN N_Message (91) END;
   PROCEDURE do_spline; BEGIN N_Message (91) END;
   PROCEDURE do_sr; BEGIN N_Message (91) END;
   PROCEDURE do_status; BEGIN N_Message (91) END;
   PROCEDURE do_store; BEGIN N_Message (91) END;
   PROCEDURE do_sub; BEGIN N_Message (91) END;
   PROCEDURE do_transp; BEGIN N_Message (91) END;
 *)

   {$I AITKEN.PAS}
   {$I EULER.PAS}
   {$I EXTERN.PAS}
   {$I HELP.PAS}
   {$I LAGRANGE.PAS}
   {$I LIST.PAS}
   {$I MTXCALC.PAS}
   {$I MTXMOVE.PAS}
   {$I PLOT.PAS}
   {$I POLYTAB.PAS}
   {$I PRIM.PAS}
   {$I SPLINE.PAS}
   {$I STATUS.PAS}

(*------------------------------------------------------------------------*)

   PROCEDURE Note;
   VAR
      Cx, Cy, i  :integer;
   BEGIN
      Cursor_safe (Cx,Cy);
      lowVideo;
      Window_off;
      gotoXY (1,1);
      write ('                 ');
      gotoXY (1,1);
      FOR i:=1 TO length (Eingabe.Arg[1]) DO BEGIN
         write (copy(Eingabe.Arg[1], i, 1), ' ');
      END;
      IF Eingabe.Arg[1] = '' THEN write ('N U M E R I K');
      Window_on;
      highVideo;
      Cursor_return (Cx,Cy);
   END;

(*------------------------------------------------------------------------*)

   PROCEDURE title;
   BEGIN
      clrScr;
      Window_off;
      lowVideo;
      gotoXY (1,1);
      writeln ('N U M E R I K              Package from Jogi-Soft                  Version 2.01');
      normVideo;
      Window_on;
      clrScr;
   END;

(*------------------------------------------------------------------------*)

BEGIN
   Message_Enable:= true;
   Pivot_Enable:= true;
   Report_Enable:= true;
   Result_Enable:= true;
   new_run:= true;
   Matrizen:= nil;
   Format:= kurz;
   leave:= false;
   clrEingabe;
   Epsilon:= 0.0;

   Window_off;
   clrScr;
   lowVideo;
   writeln;
   writeln ('-------------------------------------------------------------------------------');
   gotoXY (1,24);
   writeln ('-------------------------------------------------------------------------------');
   normVideo;
   Window_on;
   gotoXY (1,5);
   writeln ('              N U M E R I K    P R O G R A M - P A C K A G E');
   writeln ('              ----------------------------------------------');
   writeln;
   writeln;
   writeln ('                               VERSION 2.01');
   writeln;
   writeln ('                   Copyright by  J.Desteffani  09.02.1987');
   writeln;
   writeln;
   writeln ('                            (HELP is available)');

   REPEAT
      get_Befehl (Eingabe);
      IF Eingabe.Art = 0
      THEN BEGIN
         clrEingabe;
      END
      ELSE BEGIN
         note;
         writeln;
         N_Message (1);
      END;
      CASE Eingabe.Bef[1] OF
      add      :do_add;
      aitken   :do_aitken;
      cholesky :do_cholesky;
      clear    :do_clear;
      del      :do_del;
      det      :do_det;
      display  :do_display;
      edit     :do_edit;
      eps      :do_eps;
      exit     :do_exit;
      eye      :do_eye;
      euler    :do_euler;
      help     :do_help;
      input    :do_input;
      inv      :do_inv;
      lagrange :do_lagrange;
      lin      :do_lin;
      list     :do_list;
      load     :do_load;
      long     :do_long;
      lr       :do_lr;
      lu       :do_lu;
      mult     :do_mult;
      no       :;
      pivot    :do_pivot;
      plot     :do_plot;
      polytab  :do_polytab;
      prim     :do_prim;
      qr       :do_qr;
      rand     :do_rand;
      rank     :do_rank;
      report   :do_report;
      result   :do_result;
      save     :do_save;
      short    :do_short;
      spline   :do_spline;
      sr       :do_sr;
      status   :do_status;
      store    :do_store;
      sub      :do_sub;
      transp   :do_transp;
      END;
      clrEingabe;
      note;
      writeln;
      write ('.');
   UNTIL leave;
   clrScr;
   N_Message (99);
   Window_off;
   gotoXY (1, Enter_Line);
   delLine;
   gotoXY (1, Enter_Line -1);
END.