(*************************************************************************
*                                                                        *
*    program  SPLINE  from unit  NUMERIK                                 *
*                                                                        *
**************************************************************************
    *                                                                    *
    *  J. Desteffani IA85                     01.01.1987 - 04.02.1987    *
    *                                                                    *
    *  p u r p o s e :                                                   *
    *    Fuehrt die Spline - Interpolation einer angegebenen Tabelle     *
    *    aus.                                                            *
    *    (siehe Numerik - 7.1.6)                                         *
    *                                                                    *
    *  e n t r i e s :                                                   *
    *    PROCEDURE do_spline;                                            *
    *                                                                    *
    **********************************************************************)

PROCEDURE do_spline;

   TYPE
      T_Vektor = array [1..max_Ind] of real;

   VAR
      of_course               :boolean;
      i, n, s, nm1, min, max,
      Index                   :integer;
      t, low, high, nx, Step  :real;
      b, c, d, x, y           :T_Vektor;


   FUNCTION get_Wert (Punkt :real): real;
   BEGIN
      IF (Punkt <= x[1]) or (Punkt >= x[n-1]) THEN BEGIN
         IF (Punkt <= x[1])
         THEN min:= 1
         ELSE min:= n - 1;
      END
      ELSE BEGIN
         min:= 1;
         max:= n;
         REPEAT
            i:= (max + min) div 2;
            IF x[i] < Punkt
            THEN min:= i
            ELSE max:= i;
         UNTIL min+1 = max;
      END;
      nx:= (Punkt - x[min]);
      get_Wert:= y[min] + nx*(b[min] + nx*(c[min] + nx*(d[min])));
   END;


BEGIN
   load_Intern (Eingabe.Nr[2], M1);
   IF (M1.m <> 2) THEN BEGIN
      N_Error (67);
      exit;
   END;
   IF (M1.n < 2) THEN BEGIN
      N_Error (68);
      exit;
   END;
   For i:= 1 TO M1.n - 1 DO BEGIN
      IF M1.x[i,1] >= M1.x[i+1,1] THEN BEGIN
         N_Error (69);
         exit;
      END;
   END;
   FOR i:= 1 TO M1.n DO BEGIN
      x[i]:= M1.x[i,1];
      y[i]:= M1.x[i,2];
   END;
   n:= M1.n;
   IF n < 3 THEN BEGIN
      b[1]:= (y[2]-y[1])/(x[2]-x[1]);
      c[1]:= 0;
      d[1]:= 0;
      b[2]:= b[1];
      c[2]:= 0;
      d[2]:= 0;
   END
   ELSE BEGIN
      nm1:= n-1;
      d[1]:= x[2]-x[1];
      c[2]:= (y[2]-y[1])/d[1];
      FOR i:=2 TO NM1 DO BEGIN
         d[i]:= x[i+1]-x[i];
         b[i]:= 2*(d[i-1]+d[i]);
         c[i+1]:= (y[i+1]-y[i])/d[i];
         c[i]:= c[i+1]-c[i];
      END;
      b[1]:= -d[1];
      b[n]:= -d[n-1];
      c[1]:= 0;
      c[n]:= 0;
      IF n > 3 THEN BEGIN
         c[1]:= c[3]/(x[4]-x[2])-c[2]/(x[3]-x[1]);
         c[n]:= c[n-1]/(x[n]-x[n-2])-c[n-2]/(x[n-1]-x[n-3]);
         c[1]:= c[1]*d[1]*d[1]/(x[4]-x[1]);
         c[n]:= -c[n]*d[n-1]*d[n-1]/(x[n]-x[n-3]);
      END;
      FOR I:=2 TO n DO BEGIN
         t:= d[i-1]/b[i-1];
         b[i]:= b[i]-t*d[i-1];
         c[i]:= c[i]-t*c[i-1];
      END;
      c[n]:= c[n]/b[n];
      FOR i:= nm1 DOWNTO 1 DO BEGIN
         c[i]:= (c[i]-d[i]*c[i+1])/b[i];
      END;
      b[n]:= (y[n]-y[nm1])/d[nm1]+d[nm1]*(c[nm1]+2*c[n]);
      FOR i:=1 TO nm1 DO BEGIN
         b[i]:= (y[i+1]-y[i])/d[i]-d[i]*(c[i+1]+2*c[i]);
         d[i]:= (c[i+1]-c[i])/d[i];
         c[i]:= 3*c[i];
      END;
      c[n]:= 3*c[n];
      d[n]:= d[n-1];
   END;

   M2.Name:= Result_Name;
   M2.n:= M1.n -1;
   M2.m:= 3;
   FOR i:= 1 TO M1.n -1 DO BEGIN
      M2.x[i,1]:= d[i];
      M2.x[i,2]:= c[i];
      M2.x[i,3]:= b[i];
   END;

   low:= N_real_Ask (10);
   high:= N_real_Ask (11);
   Index:= N_integer_Ask (10);
   T1.Name:= Result_Name;
   T1.n:= Index;
   T1.m:= 2;
   Step:= (high - low)/(Index - 1);
   FOR s:= 1 TO Index DO BEGIN
      T1.x[s,1]:= low;
      T1.x[s,2]:= get_Wert (low);
      low:= low + Step;
   END;
   save_Intern_Tabelle (T1);
   N_Message (9);
END;
